%{
#open "syntaxe_commune";;
#open "syntaxe_miniml";;

let rec make_fun idl e = match idl with
  []      -> e
| x::rest -> MLFun (x,make_fun rest e)
;;

let make_app f el =
  let rec app_rec e = function
    []      -> e
  | a::rest -> app_rec (MLApp (e,a)) rest
  in app_rec f el
;;

let rec make_prim prim n =
  if n=0 then MLPrim prim else MLFun ("",make_prim prim (n-1))
;;

%}

%token LPAR
%token RPAR
%token COLONEQUAL
%token EQUAL
%token PLUS
%token MINUS
%token TIMES
%token DIVIDE
%token BACKQUOTE
%token DASHSUP
%token SEMI
%token SEMISEMI
%token <string> IDENT
%token <int> INT
%token <float> FLOAT
%token <string> STRING
%token <char> CHAR
%token <bool> BOOL
%token EOF
%token LET
%token REC
%token AND
%token IN
%token FUN
%token IF
%token THEN
%token ELSE
%token DO
%token RETURN

%left PLUS MINUS
%left TIMES DIVIDE

%start Programme
%type <syntaxe_miniml__programme_miniml> Programme

%%

Programme:
    Phrase SEMISEMI Programme	{ $1::$3 }
  | EOF				{ [] }
;

Phrase:
    Expr0			{ MLExpr $1 }
  | LET Liaisons		{ MLLet $2 }
  | LET REC Liaisons		{ MLLetRec $3 }

Liaisons:
    Liaison AND Liaisons	{ $1::$3 }
  | Liaison			{ [$1] }

Liaison:
    IDENT EQUAL Expr0		{ ($1,$3) }
  | IDENT Idents EQUAL Expr0	{ ($1,make_fun $2 $4) }

Exprs:
    Expr3 Exprs			{ $1::$2 }
  | Expr3			{ [$1] }

Expr3:
    LPAR Expr0 RPAR		{ $2 }
  | IDENT 			{ MLVar $1 }
  | CHAR			{ MLCst (Char $1) }
  | INT				{ MLCst (Int $1) }
  | FLOAT			{ MLCst (Float $1) }
  | STRING			{ MLCst (String $1)}
  | BOOL			{ MLCst (Bool $1) }
  | LPAR RPAR                   { MLCst Unit }

Expr2:
    Expr3			{ $1 }
  | Expr2 PLUS Expr2		{ make_app (make_prim Plus 2) [$1;$3] }
  | Expr2 MINUS Expr2		{ make_app (make_prim Moins 2) [$1;$3] }
  | Expr2 TIMES Expr2		{ make_app (make_prim Fois 2) [$1;$3] }
  | Expr2 DIVIDE Expr2		{ make_app (make_prim Divise 2) [$1;$3] }

Expr1:
    Expr2			{ $1 }
  | Expr2 EQUAL Expr2		{ make_app (make_prim Egal 2) [$1;$3] }

Expr0:
    Expr1			{ $1 }
  | Expr3 Exprs			{ make_app $1 $2 }
  | FUN Idents DASHSUP Expr0	{ make_fun $2 $4 }
  | LET Liaisons IN Expr0	{ MLLetIn ($2,$4) }
  | LET REC Liaisons IN Expr0	{ MLLetRecIn ($3,$5) }
  | IF Expr0 THEN Expr0 ELSE Expr0	{ MLIf ($2,$4,$6) }
  | IDENT COLONEQUAL Expr0	{ MLSet ($1,$3) }
  | DO Sequence RETURN Expr0    { MLSeq ($2,$4) }

Sequence:
    Expr0			{ [$1] }
  | Expr0 SEMI Sequence		{ $1::$3 }

Idents:
    IDENT			{ [$1] }
  | IDENT Idents		{ $1::$2 }






